/*---------------------------------------------------------
Implementação: TAD Árvore Binária de Busca
-----------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

typedef char ItemArv[50];
#include "itemString.h"
#include "arvBusca.h"

/* ----------------------------------------------------- */
/* Impressão da árvore */
void escreveNodoInterno( ItemArv v, int h ){
  int i;

  for( i=0; i<h; i++ )
    printf( "   " );
  write( v );
  printf( "\n" );
}

void escreveNodoExterno( int h ){
  int i;

  for( i=0; i<h; i++ )
    printf( "   " );
  printf( "*\n" );
}

void escreveNodo( ArvBusca p, int h ){
  if( p == NULL ){
    escreveNodoExterno( h );
    return;
  }
  escreveNodo( p->dir, h+1 );
  escreveNodoInterno( p->item, h );
  escreveNodo( p->esq, h+1 );
}

void escreveArv( ArvBusca arv ){
  escreveNodo( arv, 0 );
}

/* ----------------------------------------------------- */
/* Função de inicialização de uma árvore vazia */
void criaArv(ArvBusca *arv){
  *arv = NULL;
}

/* ----------------------------------------------------- */
/* Função que verifica se a árvore está vazia */
int arvVazia(ArvBusca arv){
  return (arv == NULL);
}

/* ----------------------------------------------------- */
/* Retorna a quantidade de nodos internos na arvore      */
int contaNoArv( ArvBusca p ){
    if( p == NULL )
        return 0;
    return p->n;
}

/* ----------------------------------------------------- */
/* Retorna a altura da arvore com raiz em p */
int alturaArv( ArvBusca p ){
    int he, hd;

    if( p == NULL )
        return 0;
    he= alturaArv( p-> esq );
    hd= alturaArv( p->dir );
    if( he > hd )
        return he+1;
    else
        return hd+1;
}


/* ----------------------------------------------------- 
   Verifica se a árvore está completa
*/
int arvCompleta( ArvBusca arv ){
  if( contaNoArv( arv ) == pow(2, alturaArv(arv))-1 )
    return 1;
  return 0;
}

/* ----------------------------------------------------- */
/* Criação de um novo nodo com o valor do Item preenchido com v */
ArvBusca criaNoArv( ItemArv v, ArvBusca esq, ArvBusca dir, int n ){
  ArvBusca p;

  p = (ArvBusca)malloc( sizeof(Nodo) );
  cp(p->item, v); p->esq = esq; p->dir = dir; p->n= n;
  return p;
}

/* ----------------------------------------------------- 
/* Insere um novo Item na árvore como folha */
ArvBusca insereArv( ItemArv v, ArvBusca arv ){
  if( arvVazia( arv ))
    return criaNoArv( v, NULL, NULL, 1 );
  (arv->n)++;
  if( lt( v, arv->item ))
    arv->esq= insereArv( v, arv->esq );
  else
    arv->dir= insereArv( v, arv->dir );
  return arv;
}

/* ----------------------------------------------------- 
/* Insere um novo Item na árvore como folha - função iterativa */
ArvBusca insereArvIterativo( ItemArv v, ArvBusca raiz ){
  ArvBusca p, pai;
  
  if( arvVazia( raiz ))
    return criaNoArv( v, NULL, NULL, 1 );
  p= raiz;
  while( p != NULL ){
    pai= p;
    (p->n)++;
    if( lt( v, p->item ))
      p= p->esq;
    else
      p= p->dir;
  }
  if( lt( v, pai->item))
    pai->esq= criaNoArv( v, NULL, NULL, 1 );
  else
    pai->dir= criaNoArv( v, NULL, NULL, 1 );
  return raiz;
}      


/* -----------------------------------------------------*/
/* Busca o  nodo que contem uma chave como Item */
ArvBusca buscaArv( ItemArv v, ArvBusca p ){
}        
 

/* ----------------------------------------------------- */
/* Libera espaço alocado para toda a árvore              */
void freeArv( ArvBusca p ){
  if( p != NULL ){
    freeArv( p->esq );
    freeArv( p->dir );
    free( p );
  }
}

/* ----------------------------------------------------- */
/* Busca a nodo com a k-ésima chave no árvore            */
 
ArvBusca buscaElementoK( int k, ArvBusca p ){
}

/* ----------------------------------------------------- */
/* Visita a árvore de forma ordenada                     */

void visitaOrdenado( ArvBusca p, void (*funcao)(Item) ){
}


